#!/usr/bin/env bash
#

# ./aws-deploy.sh -r everywareskeleton.local,ewtest.infomaker.io -f startup.sql -p lab-awsdeploy -s lab-ew -e eu-west-1

# ./aws-deploy.sh -e us-east-1 -s everywarestarterkit.local -r stage.ew.kimtestus.infomaker.io -f /Users/kim/Downloads/Everyware-Starter-Kit-3.0/mysql/everyware-dump-20190110.sql -p lab-awsdeploy -n lab-ew -c lab


# VARS
SCRIPT_NAME=$0
SCRIPT_NAME_REL=`dirname "$0"`
SCRIPT_PATH_ABS=`( cd "${SCRIPT_NAME_REL}" && pwd )`
SCRIPT_RANDOM_NAME="$(uuidgen | awk '{print tolower(substr($0,1,6))}')"

DEBUG_LOG="${SCRIPT_PATH_ABS}/deploy-debug.txt"

MYSQL_PASSWORD="backup"
MYSQL_WAIT_TIMEOUT="3600"
MYSQL_DATABASE="everyware"
MYSQL_VERSION="5.7.25"
MYSQL_TEMP_PATH="/tmp/mysql-backup"

DOCKER_REPO="infomakerscandinaviaab"

DOCKER_MYSQL_IMAGE="mysql:${MYSQL_VERSION}"
DOCKER_MYSQL_SERVER_NAME="mysql-server"
DOCKER_MYSQL_BACKUP_NAME="mysql-backup"

DOCKER_PHP_VERSION="7.2.27"
DOCKER_PHP_IMAGE="${DOCKER_REPO}/php-fpm:${DOCKER_PHP_VERSION}"
DOCKER_PHP_NAME="php-rewrite"

DOCKER_AWSEBCLI_IMAGE="python:3.5"
DOCKER_AWSEBCLI_NAME="awsebcli"

DOCKER_NODE_IMAGE="node:lts-alpine"
DOCKER_NODE_NAME="node"

DOCKER_CONTAINERS="${DOCKER_MYSQL_SERVER_NAME} ${DOCKER_MYSQL_BACKUP_NAME} ${DOCKER_PHP_NAME} ${DOCKER_AWSEBCLI_NAME} ${DOCKER_NODE_NAME}"
DOCKER_CONTAINERS_FIND="${DOCKER_MYSQL_SERVER_NAME}\\|${DOCKER_MYSQL_BACKUP_NAME}\\|${DOCKER_PHP_NAME}\\|${DOCKER_AWSEBCLI_NAME}\\|${DOCKER_NODE_NAME}"


# -----------------------------------------------

function Usage {
    echo "Usage:  ${SCRIPT_NAME} [-e region] [-s domain] [-r domain] [-f path] [-p profile] [-n stackname] [-c prefix]"

    echo "    -e  AWS CLI region."
    echo "    -s  Domain to search for in database."
    echo "    -r  Domain to replace with in database."
    echo "    -f  The path to the MySQL dump."
    echo "    -p  AWS CLI profile name."
    echo "    -n  AWS CloudFormation StackName."
    echo "    -c  Customer environment prefix."
    echo "    -l  Wordpress language (SE, DK, US)"
    exit 0
}

# Loop through all arguments
while getopts "r:s:f:p:n:e:c:l:" option; do
    case $option in
        s)
            DOMAIN_SEARCH="${OPTARG}"
            if [[ ! ${DOMAIN_SEARCH} == *[.]* ]]; then
                echo "[ERROR] ${DOMAIN_SEARCH} is not a valid domain!"
                Usage
                exit 1
            fi
            ;;
        r)
            DOMAIN_REPLACE="${OPTARG}"
            if [[ ! ${DOMAIN_REPLACE} == *[.]* ]]; then
                echo "[ERROR] ${DOMAIN_REPLACE} is not a valid domain!"
                Usage
                exit 1
            fi
            ;;
        f)
            MYSQL_DUMP_PATH="$(realpath ${OPTARG})"
            if [[ ! -f ${MYSQL_DUMP_PATH} ]]; then
                if [[ -f ${SCRIPT_PATH_ABS}/mysql/${MYSQL_DUMP_PATH##*/} ]]; then
                    MYSQL_DUMP_PATH="${SCRIPT_PATH_ABS}/mysql/${MYSQL_DUMP_PATH##*/}"
                else
                    echo "[ERROR] The file ${MYSQL_DUMP_PATH} could not be found!"
                    Usage
                    exit 1
                fi
            fi
            ;;
        p)
            PROFILE="${OPTARG}"
            if AWS_BIN="$(type -p "aws")" || [[ -z ${AWS_BIN} ]]; then
                if [[ -f "${HOME}/.aws/credentials" ]]; then
                    PROFILE_CHECK=$(cat ${HOME}/.aws/credentials | grep "${PROFILE}" | grep -c "\[")
                    if [[ ${PROFILE_CHECK} -eq 1 ]]; then
                        AWS_PROFILE=${PROFILE}
                    elif [[ ${PROFILE_CHECK} -gt 1 ]]; then
                        echo "[ERROR] Found more than one profile named ${PROFILE} in ${HOME}/.aws/credentials!"
                        Usage
                        exit 1
                    else
                        echo "[ERROR] Could not find the profile named ${PROFILE} in ${HOME}/.aws/credentials!"
                        Usage
                        exit 1
                    fi
                else
                    echo "[ERROR] Could not find aws-cli credentials file at ${HOME}/.aws/credentials!"
                    echo "[ERROR] Please install and configure aws-cli."
                    Usage
                    exit 1
                fi
            else
                echo "[ERROR] You must install aws-cli to be able to run this script!"
                Usage
                exit 1
            fi
            ;;
        n)
            AWS_STACK_NAME="${OPTARG}"
            if ! JQ_BIN="$(type -p "jq")" || [[ -z ${JQ_BIN} ]]; then
                echo "[ERROR] You must install jq to be able to run this script!"
                Usage
                exit 1
            fi
            ;;
        e)
            AWS_REGION="${OPTARG}"
            # Set the correct ElasticBeanstalk HostedZoneId for the region
            # http://docs.aws.amazon.com/general/latest/gr/rande.html#elasticbeanstalk_region
            case ${AWS_REGION} in
                eu-west-1)
                    BEANSTALK_HOSTED_ZONE_ID="Z2NYPWQ7DFZAZH"
                    ;;
                us-east-2)
                    BEANSTALK_HOSTED_ZONE_ID="Z14LCN19Q5QHIC"
                    ;;
                us-east-1)
                    BEANSTALK_HOSTED_ZONE_ID="Z117KPS5GTRQ2G"
                    ;;
                us-west-1)
                    BEANSTALK_HOSTED_ZONE_ID="Z1LQECGX5PH1X"
                    ;;
                us-west-2)
                    BEANSTALK_HOSTED_ZONE_ID="Z38NKT9BP95V3O"
                    ;;
                eu-north-1)
                    BEANSTALK_HOSTED_ZONE_ID="Z23GO28BZ5AETM"
                    ;;
                *)
                    BEANSTALK_HOSTED_ZONE_ID="not supported"
                    ;;
            esac

            # Exit when using a region not supported
            if [[ ${BEANSTALK_HOSTED_ZONE_ID} = "not supported" ]]; then
                echo "[ERROR] The region ${AWS_REGION} is not supported!"
                Usage
                exit 1
            fi
            ;;
        c)
            # Accept only lowercase a-z characters as CUSTOMER_PREFIX
            if [[ "${OPTARG}" =~ ^[a-z]*$ ]]; then
                CUSTOMER_PREFIX="${OPTARG}"
                ENVIRONMENT_NAME="${CUSTOMER_PREFIX}-stage-${SCRIPT_RANDOM_NAME}"
            else
                echo "[ERROR] Invalid customer prefix provided (-c ${OPTARG})"
                Usage
                exit 1
            fi
            ;;
        l)
            # Set the language for bin/wordpress-install.sh
            LANG=$(echo ${OPTARG} | awk '{print toupper($0)}')
            ;;
        \?)
            echo "Invalid option: -${OPTARG}"
            Usage
            exit 1
            ;;
        h)
            Usage
            exit 0
            ;;
    esac
done
shift $(($OPTIND - 1))


# Verify that the required vars are set
if [ -z "${CUSTOMER_PREFIX}" ] || [ -z "${DOMAIN_SEARCH}" ] || [ -z "${DOMAIN_REPLACE}" ] || [ -z "${MYSQL_DUMP_PATH}" ] || [ -z "${AWS_PROFILE}" ] || [ -z "${AWS_STACK_NAME}" ] || [ -z "${AWS_REGION}" ]; then
    echo "[ERROR] Some variables has not been set correctly!"
    Usage
    exit 1
else
    # Check access to AWS and that the provided CloudFormation Stack is present.
    AWS_BIN="${AWS_BIN} --region ${AWS_REGION} --profile ${AWS_PROFILE}"
    TEST_STACK_NAME=$(${AWS_BIN} cloudformation describe-stacks --stack-name ${AWS_STACK_NAME} | ${JQ_BIN} -r '.Stacks[].StackName')
    if [[ ! "${TEST_STACK_NAME}" == "${AWS_STACK_NAME}" ]]; then
        echo "[ERROR] Not authorized with the provided credentials!"
        Usage
        exit 1
    fi

    # Set Wordpress language, default to US if not set.
    case ${LANG} in
        SE)
            WORDPRESS_LANGUAGE="se"
            ;;
        DK)
            WORDPRESS_LANGUAGE="dk"
            ;;
        US)
            WORDPRESS_LANGUAGE="us"
            ;;
        *)
            WORDPRESS_LANGUAGE="us"
            ;;
    esac

    # Verify that there's no existing RDS or Beanstalk named ${ENVIRONMENT_NAME}

fi

echo "[DEBUG] (${DOMAIN_SEARCH}) (${DOMAIN_REPLACE}) (${MYSQL_DUMP_PATH}) (${AWS_PROFILE}) (${AWS_REGION}) (${AWS_STACK_NAME}) (${ENVIRONMENT_NAME}) (${WORDPRESS_LANGUAGE})"
echo "[DEBUG] I will search for \"${DOMAIN_SEARCH}\" and replace it with \"${DOMAIN_REPLACE}\" in database!"

rm -f ${DEBUG_LOG}



# Function to check if VAR is a domain without slashes.
function DomainCheck {
    HOSTNAME=${1}
    if [[ ! ${HOSTNAME} == *.* ]] ||
        [[ ${HOSTNAME} == http*: ]] ||
        [[ ${HOSTNAME} == */* ]] ||
        [[ ${HOSTNAME} == "" ]]; then
        return 1
    else
        return 0
    fi
}

function WaitRDSAvailable () {
  local RDS_IDENTIFIER="${1}"
  COUNT=1
  SLEEP=60
  until [[ $(${AWS_BIN} rds describe-db-instances --db-instance-identifier ${RDS_IDENTIFIER} | ${JQ_BIN} -r .DBInstances[].DBInstanceStatus) == "available" ]]; do
    if [[ ${COUNT} -eq 720 ]]; then
      echo "[ERROR] (WaitRDSAvailable) Will not wait for ${RDS_IDENTIFIER} to become available any longer ($((${COUNT}*${SLEEP})) seconds)!"
      exit 1
    fi
    sleep ${SLEEP}
    ((COUNT++))
  done
}

function WaitEBAvailable () {
  local EB_NAME="${1}"
  COUNT=1
  SLEEP=10

  until [[ $(${AWS_BIN} elasticbeanstalk describe-environments --environment-names ${EB_NAME} | ${JQ_BIN} -r .Environments[].Status) == "Ready" ]]; do
    if [[ ${COUNT} -eq 120 ]]; then
      echo "[ERROR] (WaitEBAvailable) Will not wait for ${EB_NAME} to become ready any longer ($((${COUNT}*${SLEEP})) seconds)!"
      exit 1
    fi
    sleep ${SLEEP}
    ((COUNT++))
  done
}

function WaitACMValidationConfig () {
  local ARN="${1}"
  COUNT=1
  SLEEP=2
  until [[ $(${AWS_BIN} acm describe-certificate --certificate-arn ${ARN} | ${JQ_BIN} -r .Certificate.DomainValidationOptions[1].ValidationStatus | egrep "PENDING_VALIDATION|SUCCESS") ]]; do
    if [[ ${COUNT} -eq 150 ]]; then
      echo "[ERROR] (WaitACMValidationConfig) Can't wait for PENDING_VALIDATION configuration on certificate ${ARN} any longer ($((${COUNT}*${SLEEP})) seconds)!"
      exit 1
    fi
    sleep ${SLEEP}
    ((COUNT++))
  done
}

function WaitACMIssued () {
  local ARN="${1}"
  COUNT=1
  SLEEP=2
  until [[ $(${AWS_BIN} acm describe-certificate --certificate-arn ${ARN} | ${JQ_BIN} -r .Certificate.Status) == "ISSUED" ]]; do
    if [[ ${COUNT} -eq 3600 ]]; then
      echo "[ERROR] (WaitACMIssued) Will not wait for certificate ${ARN} to become issued any longer ($((${COUNT}*${SLEEP})) seconds)!"
      exit 1
    fi
    sleep ${SLEEP}
    ((COUNT++))
  done
}



# Remove conflicting containers if they exists.
if [[ ! -z $(docker ps -a | grep "$(echo ${DOCKER_CONTAINERS_FIND})") ]]; then
    echo "[WARN] There's running conflicting containers."
    echo "[WARN] Trying to stop and remove the containers."
    docker stop ${DOCKER_CONTAINERS} > /dev/null 2>&1
    docker rm -f ${DOCKER_CONTAINERS} > /dev/null 2>&1
    if [[ ! -z $(docker ps -a | grep "$(echo ${DOCKER_CONTAINERS_FIND})") ]]; then
        echo "[ERROR] Failed to stop and delete the containers!"
        exit 1
    else
        echo "[INFO] Successfully stopped and removed the containers!"
    fi
fi



# Refresh the docker images
for DOCKER_IMAGE in $(echo ${DOCKER_MYSQL_IMAGE} ${DOCKER_PHP_IMAGE} ${DOCKER_AWSEBCLI_IMAGE} ${DOCKER_NODE_IMAGE}); do
    echo "[INFO] Refreshing image ${DOCKER_IMAGE}"
    docker pull ${DOCKER_IMAGE} > /dev/null 2>&1
done



# Run containers
echo "[INFO] Starting MySQL container!"
# Prepare the temp folder for MySQL data before starting the container
rm -rf ${MYSQL_TEMP_PATH} && mkdir -p ${MYSQL_TEMP_PATH}
#DOCKER_MYSQL_ID=$(docker run -d --rm --name ${DOCKER_MYSQL_SERVER_NAME} -v ${SCRIPT_PATH_ABS}/mysql:/docker-entrypoint-initdb.d -v ${MYSQL_TEMP_PATH}:/var/lib/mysql -e MYSQL_ROOT_PASSWORD="PassWord" -e MYSQL_DATABASE="${MYSQL_DATABASE}" ${DOCKER_MYSQL_IMAGE})
DOCKER_MYSQL_ID=$(docker run -d --rm --name ${DOCKER_MYSQL_SERVER_NAME} -v ${MYSQL_DUMP_PATH}:/docker-entrypoint-initdb.d/mysqldump.sql -v ${SCRIPT_PATH_ABS}/mysql/grants.sql:/docker-entrypoint-initdb.d/grants.sql -v ${MYSQL_TEMP_PATH}:/var/lib/mysql -e MYSQL_ROOT_PASSWORD="PassWord" -e MYSQL_DATABASE="${MYSQL_DATABASE}" ${DOCKER_MYSQL_IMAGE})

if [[ $? -ne "0" ]]; then
    echo "[ERROR] Could not start MySQL container!"
    # Proceed to removing containers
else
    echo "[INFO] Waiting for SQL import to finish!"
    COUNT=1
    until [[ $(docker logs ${DOCKER_MYSQL_SERVER_NAME} --tail 50 2>&1 | grep "Ready for start up") ]]; do
        if [[ ${COUNT} -gt "${MYSQL_WAIT_TIMEOUT}" ]]; then
          echo "[ERROR] Not finished after ${MYSQL_WAIT_TIMEOUT} seconds. Breaking out!"
          break
        fi
    sleep 10
    ((COUNT++))
    done
    echo "[INFO] Done in ${COUNT} seconds!"


    # Remove all post_type='article' from the database dump, before performing the rewrite
    # Create .my.cnf file inside MySQL container
    docker exec ${DOCKER_MYSQL_SERVER_NAME} bash -c "echo '[mysql]' > /root/.my.cnf"
    docker exec ${DOCKER_MYSQL_SERVER_NAME} bash -c "echo 'user=backup' >> /root/.my.cnf"
    docker exec ${DOCKER_MYSQL_SERVER_NAME} bash -c "echo 'password=${MYSQL_PASSWORD}' >> /root/.my.cnf"

    WP_POST_TABLES=$(docker exec ${DOCKER_MYSQL_SERVER_NAME} mysql --skip-column-names -h localhost ${MYSQL_DATABASE} -e "select blog_id from wp_blogs where not blog_id='1';" 2>&1)
    PRETTY_WP_POST_TABLES="$(echo ${WP_POST_TABLES} | sed ':a;N;$!ba;s/\n/ /g')"

    echo "[INFO] Deleting articles and comments from site id's ${PRETTY_WP_POST_TABLES}"
    for SITE_ID in ${WP_POST_TABLES}; do
        # Delete post_type article and relationships
        docker exec ${DOCKER_MYSQL_SERVER_NAME} mysql -h localhost ${MYSQL_DATABASE} -e "DELETE a,b,c FROM wp_${SITE_ID}_posts a LEFT JOIN wp_${SITE_ID}_term_relationships b ON (a.ID = b.object_id) LEFT JOIN wp_${SITE_ID}_postmeta c ON (a.ID = c.post_id) WHERE a.post_type = 'article';"
        # Delete ALL comments
        docker exec ${DOCKER_MYSQL_SERVER_NAME} mysql -h localhost ${MYSQL_DATABASE} -e "DELETE FROM wp_${SITE_ID}_comments;"
    done


    # Start the PHP container, and download Search-Replace
    echo "[INFO] Starting PHP-FPM container!"
    DOCKER_PHP_ID=$(docker run -d --rm --name ${DOCKER_PHP_NAME} --link ${DOCKER_MYSQL_SERVER_NAME}:${DOCKER_MYSQL_SERVER_NAME} ${DOCKER_PHP_IMAGE})

    if [[ $? -ne "0" ]]; then
        echo "[ERROR] Could not start PHP-FPM container!"
        # Proceed to removing containers
    else
        # Download the Search-Replace PHP script to /var/www in the container.
        docker exec ${DOCKER_PHP_NAME} curl -sSL https://github.com/interconnectit/Search-Replace-DB/archive/3.1.zip -o 3.1.zip
        docker exec ${DOCKER_PHP_NAME} unzip -qq 3.1.zip

        # Execute srdb.cli.php in ${DOCKER_PHP_NAME}
        docker exec ${DOCKER_PHP_NAME} php Search-Replace-DB-3.1/srdb.cli.php -u backup -n ${MYSQL_DATABASE} -p ${MYSQL_PASSWORD} -h ${DOCKER_MYSQL_SERVER_NAME} -s "${DOMAIN_SEARCH}" -r "${DOMAIN_REPLACE}"  >> ${DEBUG_LOG} 2>&1

        # Stop the PHP container
        docker stop ${DOCKER_PHP_NAME} >> /dev/null 2>&1
    fi


    # Start the Percona Xtrabackup container to create the RDS restore backup files.
    echo "[INFO] Starting Percona Xtrabackup container!"
    if [[ -d ${SCRIPT_PATH_ABS}/mysql/percona_backup ]]; then
        rm -rf ${SCRIPT_PATH_ABS}/mysql/percona_backup/*
        sleep 2
    fi

    docker run --rm --name ${DOCKER_MYSQL_BACKUP_NAME} --link ${DOCKER_MYSQL_SERVER_NAME}:${DOCKER_MYSQL_SERVER_NAME} --volumes-from=${DOCKER_MYSQL_SERVER_NAME} -v ${SCRIPT_PATH_ABS}/mysql/percona_backup:/xtrabackup_backupfiles perconalab/percona-xtrabackup:2.4 xtrabackup --backup --host=${DOCKER_MYSQL_SERVER_NAME} --user=backup --password=${MYSQL_PASSWORD} >> ${DEBUG_LOG} 2>&1
    if [[ $? -ne "0" ]]; then
        echo "[ERROR] Something went wrong when running Percona Xtrabackup script!"
    else
        # Upload the backup to Amazon S3
        MYSQL_BACKUP_BUCKET=$(${AWS_BIN} cloudformation list-stack-resources --stack-name ${AWS_STACK_NAME} | jq -r '.StackResourceSummaries[] | select(.LogicalResourceId == "EwDeployUserBucket") | .PhysicalResourceId')
        echo "[INFO] Copying the backup to S3Bucket ${MYSQL_BACKUP_BUCKET}!"
        ${AWS_BIN} s3 cp --recursive ${SCRIPT_PATH_ABS}/mysql/percona_backup s3://${MYSQL_BACKUP_BUCKET} >> ${DEBUG_LOG} 2>&1
        if [[ $? -ne "0" ]]; then
            echo "[ERROR] Something went wrong when copying the Percona Xtrabackup backup to S3!"
        else
            # Create the RDS Instance from the Percona Xtrabackup
            RDS_INSTANCE_USER="everyware"
            RDS_INSTANCE_PASS="$(uuidgen | awk '{print tolower(substr($0,1,20))}')"
            AWS_PARENT_STACK_NAME=$(${AWS_BIN} cloudformation describe-stacks --stack-name ${AWS_STACK_NAME} | ${JQ_BIN} -r '.Stacks[].Parameters[] | select(.ParameterKey == "NetworkStackNameStage") | .ParameterValue')
            AWS_IAM_ROLE_ARN=$(${AWS_BIN} cloudformation describe-stacks --stack-name ${AWS_STACK_NAME} | jq -r '.Stacks[].Outputs[] | select(.OutputKey == "DeployUserRoleArn") | .OutputValue')
            AWS_RDS_SECURITY_GROUP=$(${AWS_BIN} cloudformation describe-stack-resources --stack-name ${AWS_STACK_NAME} | ${JQ_BIN} -r '.StackResources[] | select(.LogicalResourceId == "EwStageRdsSecurityGroup") | .PhysicalResourceId')
            AWS_RDS_SUBNET_GROUP=$(${AWS_BIN} cloudformation describe-stack-resources --stack-name ${AWS_PARENT_STACK_NAME} | ${JQ_BIN} -r '.StackResources[] | select(.LogicalResourceId == "RdsDbSubnetGroup") | .PhysicalResourceId')

            echo "[INFO] Creating RDS instance ${ENVIRONMENT_NAME} with password ${RDS_INSTANCE_PASS}"
            ${AWS_BIN} rds restore-db-instance-from-s3 \
                       --db-instance-identifier ${ENVIRONMENT_NAME} \
                       --db-subnet-group-name ${AWS_RDS_SUBNET_GROUP} \
                       --vpc-security-group-ids ${AWS_RDS_SECURITY_GROUP} \
                       --allocated-storage 100 \
                       --storage-type standard \
                       --db-instance-class db.t3.large \
                       --engine mysql \
                       --master-username ${RDS_INSTANCE_USER} \
                       --master-user-password ${RDS_INSTANCE_PASS} \
                       --s3-bucket-name ${MYSQL_BACKUP_BUCKET} \
                       --s3-ingestion-role-arn ${AWS_IAM_ROLE_ARN} \
                       --tags Key=billing,Value=${CUSTOMER_PREFIX}-i-ew-stage Key=application,Value=everyware Key=environment,Value=stage Key=customer,Value=${CUSTOMER_PREFIX} Key=creator,Value=${SCRIPT_NAME} \
                       --source-engine mysql \
                       --no-multi-az \
                       --no-publicly-accessible \
		       --source-engine-version ${MYSQL_VERSION} > /dev/null
        fi
    fi
fi




# Create a python container to run the ElasticBeanstalk commands from.
# Link your own AWS credentials into the container.
echo "[INFO] Starting and installing the awsebcli python container!"
docker run -d --rm --name ${DOCKER_AWSEBCLI_NAME} -v ${HOME}/.aws:/root/.aws -e AWS_EB_PROFILE=${AWS_PROFILE} ${DOCKER_AWSEBCLI_IMAGE} /bin/sh -c "while true; do sleep 60; done" > /dev/null
if [[ $? -ne "0" ]]; then
    echo "[ERROR] Something went wrong when starting the python container!"
else
    # Install awsebcli and configuration files.
    docker exec ${DOCKER_AWSEBCLI_NAME} pip install awsebcli > /dev/null
    docker exec ${DOCKER_AWSEBCLI_NAME} mkdir -p .elasticbeanstalk > /dev/null
    docker exec ${DOCKER_AWSEBCLI_NAME} /bin/sh -c "cat >.elasticbeanstalk/config.yml <<EOL
branch-defaults:
  default:
    environment: null
    group_suffix: null
global:
  application_name: ${CUSTOMER_PREFIX}-application
  default_region: ${AWS_REGION}
EOL"

    # Set some AWS stack vars
    AWS_INSTANCE_PROFILE=$(${AWS_BIN} cloudformation describe-stack-resources --stack-name ${AWS_STACK_NAME} | ${JQ_BIN} -r '.StackResources[] | select(.LogicalResourceId == "EwInstanceProfile") | .PhysicalResourceId')
    AWS_SECURITY_GROUP_ID=$(${AWS_BIN} cloudformation describe-stack-resources --stack-name ${AWS_STACK_NAME} | ${JQ_BIN} -r '.StackResources[] | select(.LogicalResourceId == "EwStageInstanceSecurityGroup") | .PhysicalResourceId')
    AWS_REDIS_HOST=$(${AWS_BIN} cloudformation describe-stacks --stack-name ${AWS_STACK_NAME} | ${JQ_BIN} -r '.Stacks[].Outputs[] | select(.OutputKey | contains("EverywareRedisEndpoint")) | .OutputValue')
    AWS_ACCESS_KEY_ID=$(${AWS_BIN} cloudformation describe-stacks --stack-name ${AWS_STACK_NAME} | ${JQ_BIN} -r '.Stacks[].Outputs[] | select(.OutputKey | contains("EverywareStaticAccessKey")) | .OutputValue')
    AWS_SECRET_KEY=$(${AWS_BIN} cloudformation describe-stacks --stack-name ${AWS_STACK_NAME} | ${JQ_BIN} -r '.Stacks[].Outputs[] | select(.OutputKey | contains("EverywareStaticSecretAccessKey")) | .OutputValue')
    AWS_DEPLOY_BUCKET=$(${AWS_BIN} cloudformation describe-stacks --stack-name ${AWS_STACK_NAME} | ${JQ_BIN} -r '.Stacks[].Outputs[] | select(.OutputKey | contains("EverywareDeployBucket")) | .OutputValue')

    AWS_PUBLIC_SUBNETS=$(${AWS_BIN} cloudformation describe-stacks --stack-name ${AWS_PARENT_STACK_NAME} | ${JQ_BIN} -r '.Stacks[].Outputs[] | select(.OutputKey | contains("PublicSubnet")) | .OutputValue' | paste -sd, -)
    AWS_PRIVATE_SUBNETS=$(${AWS_BIN} cloudformation describe-stacks --stack-name ${AWS_PARENT_STACK_NAME} | ${JQ_BIN} -r '.Stacks[].Outputs[] | select(.OutputKey | contains("PrivateSubnet")) | .OutputValue' | paste -sd, -)
    AWS_VPC_ID=$(${AWS_BIN} cloudformation describe-stacks --stack-name ${AWS_PARENT_STACK_NAME} | ${JQ_BIN} -r '.Stacks[].Outputs[] | select(.OutputKey == "VpcId") | .OutputValue')
    AWS_VPC_CIDR=$(${AWS_BIN} cloudformation describe-stacks --stack-name ${AWS_PARENT_STACK_NAME} | ${JQ_BIN} -r '.Stacks[].Outputs[] | select(.OutputKey == "VpcCidr") | .OutputValue')

    AWS_BEANSTALK_PLATFORM=$(${AWS_BIN} elasticbeanstalk list-available-solution-stacks | jq -r '.SolutionStackDetails[] | select(.SolutionStackName | contains("-container")) | .SolutionStackName' | head -1)

    # Determine which key-pair should be used for the Beanstalk environment.
    # First check if there are any keys called *ew* or *everyware*.
    # Otherwise pick the same key as mentioned in the ${AWS_PARENT_STACK_NAME} stack.
    CHECK_KEY_PAIR=$(${AWS_BIN} ec2 describe-key-pairs | ${JQ_BIN} -r '.KeyPairs[] | select((.KeyName | contains("everyware")) or select(.KeyName | contains("ew"))) | .KeyName')
    if [[ ${CHECK_KEY_PAIR} == "" ]]; then
        AWS_KEY_PAIR_NAME=$(${AWS_BIN} cloudformation describe-stacks --stack-name ${AWS_PARENT_STACK_NAME} | ${JQ_BIN} -r '.Stacks[].Parameters[] | select(.ParameterKey == "KeyName") | .ParameterValue')
    else
        AWS_KEY_PAIR_NAME=${CHECK_KEY_PAIR}
    fi

    # Creating a basic ElasticBeanstalk environment, without any ENVIRONMENT vars.
    echo "[INFO] Creating ElasticBeanstalk environment ${ENVIRONMENT_NAME}"
    docker exec ${DOCKER_AWSEBCLI_NAME} eb create ${ENVIRONMENT_NAME} \
 --tags billing=${CUSTOMER_PREFIX}-i-ew-stage,application=everyware,environment=stage,customer=${CUSTOMER_PREFIX},creator=${SCRIPT_NAME} \
-ip "${AWS_INSTANCE_PROFILE}" \
--keyname "${AWS_KEY_PAIR_NAME}" \
--elb-type "application" \
--platform "${AWS_BEANSTALK_PLATFORM}" \
--vpc.id "${AWS_VPC_ID}" \
--vpc.ec2subnets "${AWS_PRIVATE_SUBNETS}" \
--vpc.elbsubnets "${AWS_PUBLIC_SUBNETS}" \
--vpc.securitygroups "${AWS_SECURITY_GROUP_ID}" \
--vpc.elbpublic \
--sample \
--envvars \
APP_ENV=stage,\
APP_DOMAIN=${DOMAIN_REPLACE},\
APP_ORGANISATION=${CUSTOMER_PREFIX},\
AWS_ACCESS_KEY_ID=\"${AWS_ACCESS_KEY_ID}\",\
AWS_SECRET_KEY=\"${AWS_SECRET_KEY}\",\
AWS_SES_REGION=\"${AWS_REGION}\",\
AWS_SES_FROM=empty,\
CF_IMENGINE=\"https://imengine.ew.${CUSTOMER_PREFIX}.infomaker.io\",\
CF_STATIC=\"https://static.ew.${CUSTOMER_PREFIX}.infomaker.io\",\
CF_PDF=\"https://pdf.static.${CUSTOMER_PREFIX}.infomaker.io\",\
DB_NAME=\"${MYSQL_DATABASE}\",\
DB_PASSWORD=\"${RDS_INSTANCE_PASS}\",\
DB_USER=\"${RDS_INSTANCE_USER}\",\
IMHOST=\"${CUSTOMER_PREFIX}stage\",\
NEWRELIC_ENABLED=false,\
NEWRELIC_LICENSE=false,\
NGX_WORKER_CONNECTIONS=1024,\
NGX_WORKER_PROCESSES=2,\
PHP_MAX_CHILDREN=12,\
PHP_MAX_SPARE_SERVERS=8,\
PHP_MIN_SPARE_SERVERS=2,\
PHP_START_SERVERS=4,\
PHP_MAX_UPLOAD=200M,\
PHP_DEFAULT_LOCALE=sv-SE,\
PHP_TIMEZONE=Europe/Stockholm,\
PHP_REQUEST_TERMINATE_TIMEOUT=300,\
OC_URL=empty,\
OC_PORT=empty,\
OC_USER=empty,\
OC_PASSWORD=empty,\
FRONT_CACHE_TTL=\"300\",\
REDIS_FC_HOST=\"${AWS_REDIS_HOST}\",\
REDIS_OB_HOST=\"${AWS_REDIS_HOST}\",\
REDIS_OB_MAXTTL=10800,\
REDIS_OB_PREFIX=\"${CUSTOMER_PREFIX}stage\" \
--profile ${AWS_PROFILE} >> ${DEBUG_LOG} 2>&1

    # Check for errors on creating the Beanstalk environment
    if [[ $? -ne "0" ]]; then
        echo "[ERROR] Something went wrong when creating the Beanstalk environment ${ENVIRONMENT_NAME}!"
    else
        echo "[INFO] Requesting certificate for the domain ${DOMAIN_REPLACE}"
        AWS_CERTIFICATE_ARN=$(${AWS_BIN} acm request-certificate --domain-name "${DOMAIN_REPLACE}" --validation-method DNS --subject-alternative-names "*.${DOMAIN_REPLACE}" | ${JQ_BIN} -r '.CertificateArn')
        WaitACMValidationConfig "${AWS_CERTIFICATE_ARN}"
	sleep 5
        VALIDATION_STRING=$(${AWS_BIN} acm describe-certificate --certificate-arn ${AWS_CERTIFICATE_ARN} | ${JQ_BIN} -r ".Certificate.DomainValidationOptions[] | select(.DomainName == \"${DOMAIN_REPLACE}\") | [.ResourceRecord.Type,.ResourceRecord.Name,.ResourceRecord.Value] | \"\(.[0]) \(.[1]) \(.[2])\"")

        # Cut the whitespace separated string into three
        VALIDATION_TYPE="$(echo ${VALIDATION_STRING} | awk '{ print $1 }')"
        VALIDATION_NAME="$(echo ${VALIDATION_STRING} | awk '{ print $2 }')"
        VALIDATION_VALUE="$(echo ${VALIDATION_STRING} | awk '{ print $3 }')"

        AWS_HOSTED_ZONE_NAME=$(${AWS_BIN} cloudformation describe-stacks --stack-name ${AWS_PARENT_STACK_NAME} | ${JQ_BIN} -r '.Stacks[].Outputs[] | select(.OutputKey == "HostedZoneName") | .OutputValue')
        AWS_HOSTED_ZONE_ID=$(${AWS_BIN} cloudformation describe-stacks --stack-name ${AWS_PARENT_STACK_NAME} | ${JQ_BIN} -r '.Stacks[].Outputs[] | select(.OutputKey == "HostedZoneId") | .OutputValue')
        BEANSTALK_DNS_NAME=$(${AWS_BIN} elasticbeanstalk describe-environments --environment-names ${ENVIRONMENT_NAME} | ${JQ_BIN} -r '.Environments[].CNAME')

        # Create the RecordSet changes as a JSON file
        cat >/tmp/route53.json <<EOL
{
    "Changes": [
        {
            "Action": "UPSERT",
            "ResourceRecordSet": {
                "Type": "A",
                "AliasTarget": {
                    "EvaluateTargetHealth": false,
                    "HostedZoneId": "${BEANSTALK_HOSTED_ZONE_ID}",
                    "DNSName": "${BEANSTALK_DNS_NAME}"
                },
                "Name": "${DOMAIN_REPLACE}"
            }
        },
        {
            "Action": "UPSERT",
            "ResourceRecordSet": {
                "Type": "A",
                "AliasTarget": {
                    "EvaluateTargetHealth": false,
                    "HostedZoneId": "${BEANSTALK_HOSTED_ZONE_ID}",
                    "DNSName": "${BEANSTALK_DNS_NAME}"
                },
                "Name": "*.${DOMAIN_REPLACE}"
            }
        },
        {
            "Action": "UPSERT",
            "ResourceRecordSet": {
                "Type": "${VALIDATION_TYPE}",
                "Name": "${VALIDATION_NAME}",
                "TTL": 300,
                "ResourceRecords": [
                    {
                      "Value": "${VALIDATION_VALUE}"
                    }
                ]
            }
        }
    ]
}
EOL

        echo "[INFO] Creating Route53 record sets for ${DOMAIN_REPLACE}!"
        ${AWS_BIN} route53 change-resource-record-sets --hosted-zone-id ${AWS_HOSTED_ZONE_ID} --change-batch file:///tmp/route53.json >> ${DEBUG_LOG} 2>&1
        # Check for errors on creating the Route53 record set
        if [[ $? -ne "0" ]]; then
            echo "[ERROR] Something went wrong when creating the Route53 record set!"
            rm /tmp/route53.json
        else
            rm /tmp/route53.json

            # Build the Wordpress code
            echo "[INFO] Downloading Wordpress with language ${WORDPRESS_LANGUAGE}."
            /bin/bash ${SCRIPT_PATH_ABS}/bin/wordpress-install.sh --language ${WORDPRESS_LANGUAGE} --force >> ${DEBUG_LOG} 2>&1

            echo "[INFO] Installing Lua libraries with the Makefile."
            /usr/bin/make --directory ${SCRIPT_PATH_ABS} luarocks >> ${DEBUG_LOG} 2>&1

            echo "[INFO] Running composer install and building the application."
            for CMD in "install --ignore-platform-reqs --no-dev" "dump-autoload -o"; do
                docker run --rm --volume ${SCRIPT_PATH_ABS}:/app ${DOCKER_PHP_IMAGE} composer ${CMD} >> ${DEBUG_LOG} 2>&1
                RETURNCODE=$?
                if [[ $RETURNCODE != 0 ]] ; then
                    echo "\"composer ${CMD}\" exited with errorcode other than 0."
                    exit 1
                fi
            done

            # Install Node, gulp and run build command if gulpfile actually exists
            if [[ -f gulpfile.js ]]; then
                echo "[INFO] Starting and installing the node container ${DOCKER_NODE_NAME}!"
                docker run -d --rm --name ${DOCKER_NODE_NAME} -v ${SCRIPT_PATH_ABS}:/app ${DOCKER_NODE_IMAGE} /bin/sh -c "while true; do sleep 60; done" > /dev/null
                if [[ $? -ne "0" ]]; then
                    echo "[ERROR] Something went wrong when starting the node container!"
                else
                    echo "[INFO] Installing gulp-cli in ${DOCKER_NODE_NAME}."
                    docker exec ${DOCKER_NODE_NAME} npm install --global gulp-cli >> ${DEBUG_LOG} 2>&1
                    echo "[INFO] Installing npm in ${DOCKER_NODE_NAME}."
                    docker exec ${DOCKER_NODE_NAME} npm install --prefix /app >> ${DEBUG_LOG} 2>&1
                    echo "[INFO] Installing gulp in ${DOCKER_NODE_NAME}."
                    docker exec ${DOCKER_NODE_NAME} gulp --gulpfile /app/gulpfile.js >> ${DEBUG_LOG} 2>&1
                fi
            fi

            # Rewrite the Dockerrun.aws.json to use integers for memory
            sed -i '' -e "s|PHP_MEM|512|g" ${SCRIPT_PATH_ABS}/Dockerrun.aws.json
            sed -i '' -e "s|NGX_MEM|256|g" ${SCRIPT_PATH_ABS}/Dockerrun.aws.json

            # Build the zip with exclude-file
            zip -q --symlinks -r /tmp/${ENVIRONMENT_NAME}-init.zip . -x@${SCRIPT_PATH_ABS}/.exclude-from-zip
            if [[ $? -ne "0" ]]; then
                echo "[ERROR] Something went wrong when creating the zip archive!"
            else
                echo "[INFO] Verifying that both Beanstalk and ACM certificate is in a valid state."
                WaitACMIssued "${AWS_CERTIFICATE_ARN}"
                WaitEBAvailable "${ENVIRONMENT_NAME}"

                echo "[INFO] Uploading and creating application version to Beanstalk."
                ${AWS_BIN} s3 cp \
                           /tmp/${ENVIRONMENT_NAME}-init.zip \
                           s3://${AWS_DEPLOY_BUCKET}/ \
                           >> ${DEBUG_LOG} 2>&1
                ${AWS_BIN} elasticbeanstalk create-application-version \
                           --application-name ${CUSTOMER_PREFIX}-application \
                           --version-label ${ENVIRONMENT_NAME}-init \
                           --source-bundle S3Bucket=${AWS_DEPLOY_BUCKET},S3Key=${ENVIRONMENT_NAME}-init.zip \
                           >> ${DEBUG_LOG} 2>&1
                WaitEBAvailable "${ENVIRONMENT_NAME}"

                # Modify the RDSInstance according to a normal stage setup.
                # First, get some vars before starting WaitRDSAvailable
                AWS_RDS_ENGINE_VERSION=$(${AWS_BIN} rds describe-db-engine-versions --engine mysql --db-parameter-group-family mysql5.7 | ${JQ_BIN} -r '.DBEngineVersions[].EngineVersion' | tail -1)

                # Wait for restored RDS Instance to become available
                echo "[INFO] Waiting for RDS instance ${ENVIRONMENT_NAME} to become available!"
                WaitRDSAvailable ${ENVIRONMENT_NAME}

                # Modify the restored RDS Instance
                echo "[INFO] Modifying and upgrading the database ${ENVIRONMENT_NAME}"
                ${AWS_BIN} rds modify-db-instance \
                           --db-instance-identifier ${ENVIRONMENT_NAME} \
                           --engine-version ${AWS_RDS_ENGINE_VERSION} \
                           --db-instance-class db.t3.micro \
                           --storage-type standard \
                           --backup-retention-period 0 \
                           --no-multi-az \
                           --no-publicly-accessible \
                           --allow-major-version-upgrade \
                           --apply-immediately >> ${DEBUG_LOG} 2>&1

                AWS_RDS_ENDPOINT=$(${AWS_BIN} rds describe-db-instances --db-instance-identifier ${ENVIRONMENT_NAME} | ${JQ_BIN} -r '.DBInstances[].Endpoint.Address')

                echo "[INFO] Deploying ${ENVIRONMENT_NAME}-init to environment ${ENVIRONMENT_NAME}!"
                ${AWS_BIN} elasticbeanstalk update-environment \
                           --environment-name ${ENVIRONMENT_NAME} \
                           --version-label ${ENVIRONMENT_NAME}-init \
                           --option-settings \
                             Namespace=aws:elasticbeanstalk:environment:process:default,OptionName=HealthCheckPath,Value=/health.php \
                             Namespace=aws:elasticbeanstalk:application:environment,OptionName=DB_HOST,Value=${AWS_RDS_ENDPOINT} \
                             Namespace=aws:elbv2:listener:443,OptionName=Protocol,Value=HTTPS \
                             Namespace=aws:elbv2:listener:443,OptionName=ListenerEnabled,Value=true \
                             Namespace=aws:elbv2:listener:443,OptionName=SSLCertificateArns,Value=${AWS_CERTIFICATE_ARN} \
                             Namespace=aws:autoscaling:launchconfiguration,OptionName=SSHSourceRestriction,Value="$(echo "'tcp, 22, 22, ${AWS_VPC_CIDR}'")" \
                           --debug >> ${DEBUG_LOG} 2>&1
                           # https://docs.aws.amazon.com/elasticbeanstalk/latest/dg/command-options-general.html

                echo "[INFO] Waiting for environment ${ENVIRONMENT_NAME} to be available!"
                WaitEBAvailable ${ENVIRONMENT_NAME}
            fi

            # Revert the sed's
            sed -i '' -e "s|512|PHP_MEM|g" ${SCRIPT_PATH_ABS}/Dockerrun.aws.json
            sed -i '' -e "s|256|NGX_MEM|g" ${SCRIPT_PATH_ABS}/Dockerrun.aws.json

            echo "[INFO] Verifying that the RDS instance ${ENVIRONMENT_NAME} is ready!"
            WaitRDSAvailable ${ENVIRONMENT_NAME}
        fi
    fi
fi



# Finally stopping and removing containers.
for DOCKER_IMAGE in $(echo ${DOCKER_CONTAINERS}); do
    echo "[INFO] Stopping and removing container ${DOCKER_IMAGE}"
    docker stop ${DOCKER_IMAGE} > /dev/null 2>&1
    docker rm -f ${DOCKER_IMAGE} > /dev/null 2>&1
done


# Configure Jenkins deploy SSH key creation
# https://mohitgoyal.co/2017/02/27/configuring-ssh-authentication-between-github-and-jenkins/
